from django.conf import settings
from django.contrib.contenttypes.generic import GenericRelation
from django.contrib.sites.models import Site
from django.db import models
from django.utils.translation import ugettext_lazy as _
from parler.fields import TranslatedField
from parler.models import TranslatableModel, TranslatedFieldsModel
from fluent_blogs.urlresolvers import blog_reverse
from fluent_blogs.models.managers import EntryManager, TranslatableEntryManager
from fluent_blogs.utils.compat import get_user_model_name
from fluent_blogs import appsettings
from fluent_contents.models import PlaceholderField, ContentItemRelation
from parler.utils.context import switch_language

__all__ = (
    # Mixins
    'AbstractSharedEntryBaseMixin',
    'AbstractTranslatedFieldsEntryBaseMixin',
    'ExcerptEntryMixin',
    'ContentsEntryMixin',
    'CommentsEntryMixin',
    'CategoriesEntryMixin',
    'TagsEntryMixin',
    'SeoEntryMixin',

    # Untranslated base classes
    'AbstractEntryBase',
    'AbstractEntry',

    # Translated base classes.
    'AbstractTranslatableEntryBase',
    'AbstractTranslatableEntry',
    'AbstractTranslatedFieldsEntryBase',
    'AbstractTranslatedFieldsEntry',
)


# Optional tagging support
TaggableManager = None
if 'taggit_autosuggest' in settings.INSTALLED_APPS:
    from taggit_autosuggest.managers import TaggableManager
elif 'taggit_autocomplete_modified' in settings.INSTALLED_APPS:
    from taggit_autocomplete_modified.managers import TaggableManagerAutocomplete as TaggableManager
elif 'taggit' in settings.INSTALLED_APPS:
    from taggit.managers import TaggableManager


# Optional commenting support
# Make sure that django.contrib.comments is not imported in the app,
# unless it exists in INSTALLED_APPS. Otherwise, the User delete view
# breaks because it tries to delete comments associated with the user.
comments = None
moderator = None
CommentModel = None
CommentModerator = None
if 'django.contrib.comments' in settings.INSTALLED_APPS:
    from django.contrib import comments
    from django.contrib.comments.moderation import moderator, CommentModerator
    CommentModel = comments.get_model()

# Can't use EmptyQueryset stub in Django 1.6 anymore,
# using this model to build a queryset instead.
class CommentModelStub(models.Model):
    class Meta:
        managed = False
        db_table = "django_comments_stub"



class AbstractTranslatedFieldsEntryBaseMixin(models.Model):
    """
    The base translated fields.
    """
    title = models.CharField(_("Title"), max_length=200)
    slug = models.SlugField(_("Slug"))

    class Meta:
        abstract = True


class AbstractSharedEntryBaseMixin(models.Model):
    """
    The basic interface for blog entries.
    """
    #: Tagging marker for code to recognize translated models.
    is_translatable_model = False

    # Some publication states
    DRAFT = 'd'
    PUBLISHED = 'p'
    STATUSES = (
        (PUBLISHED, _('Published')),
        (DRAFT, _('Draft')),
    )

    parent_site = models.ForeignKey(Site, editable=False, default=Site.objects.get_current)

    status = models.CharField(_('status'), max_length=1, choices=STATUSES, default=DRAFT, db_index=True)
    publication_date = models.DateTimeField(_('publication date'), null=True, db_index=True, help_text=_('''When the entry should go live, status must be "Published".'''))
    publication_end_date = models.DateTimeField(_('publication end date'), null=True, blank=True, db_index=True)

    # Metadata
    author = models.ForeignKey(get_user_model_name(), verbose_name=_('author'))
    creation_date = models.DateTimeField(_('creation date'), editable=False, auto_now_add=True)
    modification_date = models.DateTimeField(_('last modification'), editable=False, auto_now=True)

    objects = EntryManager()


    class Meta:
        verbose_name = _("Blog entry")
        verbose_name_plural = _("Blog entries")
        ordering = ('-publication_date',)
        get_latest_by = "publication_date"  # Support Entry.objects.latest() call.
        abstract = True


    def __unicode__(self):
        return self.title


    def get_absolute_url(self):
        return self.default_url


    @property
    def default_url(self):
        """
        The internal implementation of :func:`get_absolute_url`.
        This function can be used when overriding :func:`get_absolute_url` in the settings.
        For example::

            ABSOLUTE_URL_OVERRIDES = {
                'fluent_blogs.Entry': lambda o: "http://example.com" + o.default_url
            }
        """
        root = blog_reverse('entry_archive_index', ignore_multiple=True, language_code=self.get_current_language())
        return root + self.get_relative_url()


    def get_relative_url(self):
        """
        Return the link path from the archive page.
        """
        # Return the link style, using the permalink style setting.
        return appsettings.FLUENT_BLOGS_ENTRY_LINK_STYLE.lstrip('/').format(
            year = self.publication_date.strftime('%Y'),
            month = self.publication_date.strftime('%m'),
            day = self.publication_date.strftime('%d'),
            slug = self.slug,
            pk = self.pk,
        )


    def get_current_language(self):
        return None  # Normal untranslated model: the API is there, but unused.


    def get_short_url(self):
        return blog_reverse('entry_shortlink', kwargs={'pk': self.pk}, ignore_multiple=True)


    @property
    def url(self):
        """
        The URL of the entry, provided for template code.
        """
        return self.get_absolute_url()


    @property
    def is_published(self):
        return self.status == self.PUBLISHED


    @property
    def is_draft(self):
        return self.status == self.DRAFT


    @property
    def previous_entry(self):
        """
        Return the previous entry
        """
        if not self.publication_date:
            # Protection for manually created models (Entry.objects.create())
            return None
        qs = self.__class__.objects.published()
        if self.is_translatable_model:
            qs = qs.translated()

        entries = qs.filter(publication_date__lt=self.publication_date).order_by('-publication_date')[:1]
        return entries[0] if entries else None


    @property
    def next_entry(self):
        """
        Return the next entry
        """
        if not self.publication_date:
            return None
        qs = self.__class__.objects.published()
        if self.is_translatable_model:
            qs = qs.translated()

        entries = qs.filter(publication_date__gt=self.publication_date).order_by('publication_date')[:1]
        return entries[0] if entries else None


class ExcerptEntryMixin(models.Model):
    """
    Mixin for adding the excerpt text to a blog entry.
    """
    intro = models.TextField(_("Introtext"))

    class Meta:
        abstract = True


class ContentsEntryMixin(models.Model):
    """
    Mixin for adding contents to a blog entry
    """
    contents = PlaceholderField("blog_contents")

    # Adding the ContentItemRelation makes sure the admin can find all deleted objects too.
    contentitem_set = ContentItemRelation()

    class Meta:
        abstract = True


class CommentsEntryMixin(models.Model):
    """
    Mixin for adding comments support to a blog entry.
    """
    enable_comments = models.BooleanField(_("Enable comments"), default=True)


    # Make association with django.contrib.comments optional
    if CommentModel is not None:
        all_comments = GenericRelation(CommentModel, verbose_name=_("Comments"), object_id_field='object_pk')
    else:
        # Provide a stub so templates don't break.
        # This avoids importing django.contrib.comments models when the app is not used.
        all_comments = CommentModelStub.objects.none()

    class Meta:
        abstract = True


    @property
    def comments(self):
        """
        Return the visible comments.
        """
        if CommentModel is None:
            # No local comments, return empty queryset.
            # The project might be using DISQUS or Facebook comments instead.
            return CommentModelStub.objects.none()
        else:
            return CommentModel.objects.for_model(self).filter(is_public=True, is_removed=False)


    @property
    def comments_are_open(self):
        """
        Check if comments are open
        """
        if not self.enable_comments or CommentModel is None:
            return False

        try:
            # Get the moderator which is installed for this model.
            mod = moderator._registry[self.__class__]
        except KeyError:
            return True

        # Check the 'enable_field', 'auto_close_field' and 'close_after',
        # by reusing the basic Django policies.
        return CommentModerator.allow(mod, None, self, None)


    @property
    def comments_are_moderated(self):
        """
        Check if comments are moderated
        """
        try:
            # Get the moderator which is installed for this model.
            mod = moderator._registry[self.__class__]
        except KeyError:
            return False

        # Check the 'auto_moderate_field', 'moderate_after',
        # by reusing the basic Django policies.
        return CommentModerator.moderate(mod, None, self, None)


class CategoriesEntryMixin(models.Model):
    """
    Mixin for adding category support to a blog entry.
    """
    categories = models.ManyToManyField(appsettings.FLUENT_BLOGS_CATEGORY_MODEL, verbose_name=_("Categories"), blank=True)

    class Meta:
        abstract = True


class TagsEntryMixin(models.Model):
    """
    Mixin for adding tags to a blog entry
    """
    # Make association with tags optional.
    if TaggableManager is not None:
        tags = TaggableManager(blank=True)
    else:
        tags = None

    class Meta:
        abstract = True


class SeoEntryMixin(models.Model):
    """
    Mixin for adding SEO fields to a blog entry.
    """
    meta_keywords = models.CharField(_('keywords'), max_length=255, blank=True, default='', help_text=_("When this field is not filled in, the the tags will be used."))
    meta_description = models.CharField(_('description'), max_length=255, blank=True, default='', help_text=_("When this field is not filled in, the contents or intro text will be used."))

    class Meta:
        abstract = True


class AbstractTranslatableEntryBase(
    TranslatableModel,
    AbstractSharedEntryBaseMixin):
    """
    The translatable abstract entry base model.
    """
    # Update tagging marker
    is_translatable_model = True

    title = TranslatedField(any_language=True)
    slug = TranslatedField()

    # Correct ordering of base classes should avoid getting the TranslatableModel.manager
    # to override the base classes. However, this caused other errors (django-parler not receiving abstract=True),
    # so kept the ordering in human-logical form, and restore the manager here.
    objects = TranslatableEntryManager()

    class Meta:
        abstract = True

    @property
    def default_url(self):
        """
        Make sure that the URL is translated in the current language.
        """
        with switch_language(self):
            return super(AbstractTranslatableEntryBase, self).default_url


class AbstractTranslatedFieldsEntryBase(
    TranslatedFieldsModel,
    AbstractTranslatedFieldsEntryBaseMixin):
    """
    The translatable fields for the base entry model.
    """
    master = None   # FK to shared model.

    class Meta:
        abstract = True


# For compatibility with old untranslated models.
class AbstractEntryBase(
    AbstractSharedEntryBaseMixin,
    AbstractTranslatedFieldsEntryBaseMixin):
    """
    The classic abstract entry base model.
    """
    # Not needed, but be explicit about the manager with this many base classes
    objects = EntryManager()

    class Meta:
        abstract = True


class AbstractEntry(
        AbstractEntryBase,
        ExcerptEntryMixin,
        ContentsEntryMixin,
        CommentsEntryMixin,
        CategoriesEntryMixin,
        TagsEntryMixin,
        SeoEntryMixin):
    """
    The classic entry model, as abstract model.
    """
    class Meta:
        abstract = True


class AbstractTranslatableEntry(
    AbstractTranslatableEntryBase,
    ContentsEntryMixin,
    CommentsEntryMixin,
    CategoriesEntryMixin,
    TagsEntryMixin):
    """
    The default entry model for translated blog posts, as abstract model.
    """
    class Meta:
        abstract = True


class AbstractTranslatedFieldsEntry(
    AbstractTranslatedFieldsEntryBase,
    ExcerptEntryMixin,
    SeoEntryMixin):
    """
    The default translated fields model for blog posts, as abstract model.
    """
    class Meta:
        abstract = True


# Make sure the 'tags' field is ignored by South
try:
    from south.modelsinspector import add_ignored_fields
except ImportError:
    pass
else:
    # South should ignore the tags field as it's a RelatedField.
    add_ignored_fields((
        "^taggit\.managers\.TaggableManager",
        "^taggit_autocomplete_modified\.managers\.TaggableManagerAutocomplete",
    ))



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'BlogPage'
        db.create_table('pagetype_blogpage_blogpage', (
            ('urlnode_ptr', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['fluent_pages.UrlNode'], unique=True, primary_key=True)),
        ))
        db.send_create_signal('blogpage', ['BlogPage'])


    def backwards(self, orm):
        # Deleting model 'BlogPage'
        db.delete_table('pagetype_blogpage_blogpage')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'blogpage.blogpage': {
            'Meta': {'ordering': "('lft', 'title')", 'object_name': 'BlogPage', 'db_table': "'pagetype_blogpage_blogpage'", '_ormbases': ['fluent_pages.Page']},
            'urlnode_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['fluent_pages.UrlNode']", 'unique': 'True', 'primary_key': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'fluent_pages.page': {
            'Meta': {'ordering': "('lft', 'title')", 'object_name': 'Page', 'db_table': "'fluent_pages_urlnode'", '_ormbases': ['fluent_pages.UrlNode'], 'proxy': 'True'}
        },
        'fluent_pages.urlnode': {
            'Meta': {'ordering': "('lft', 'title')", 'object_name': 'UrlNode'},
            '_cached_url': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '300', 'db_index': 'True', 'blank': 'True'}),
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'in_navigation': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'override_url': ('django.db.models.fields.CharField', [], {'max_length': '300', 'blank': 'True'}),
            'parent': ('fluent_pages.models.fields.PageTreeForeignKey', [], {'blank': 'True', 'related_name': "'children'", 'null': 'True', 'to': "orm['fluent_pages.UrlNode']"}),
            'parent_site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polymorphic_fluent_pages.urlnode_set'", 'null': 'True', 'to': "orm['contenttypes.ContentType']"}),
            'publication_date': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'publication_end_date': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'d'", 'max_length': '1', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['blogpage']


import django
from django.conf import settings
from django.contrib import admin
from django.contrib.admin import widgets
from django.contrib.admin.widgets import AdminTextInputWidget, AdminTextareaWidget
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.urlresolvers import NoReverseMatch
from django.forms import ModelForm
from django.utils.translation import ugettext, ugettext_lazy as _
from fluent_blogs import appsettings
from fluent_blogs.base_models import AbstractEntryBase
from fluent_blogs.models import get_entry_model, Entry_Translation
from fluent_blogs.models.query import get_date_range
from fluent_blogs.utils.compat import now
from fluent_contents.admin import PlaceholderFieldAdmin
from parler.admin import TranslatableAdmin
from parler.forms import TranslatableModelForm
from parler.models import TranslatableModel, TranslationDoesNotExist


EntryModel = get_entry_model()


class AbstractEntryBaseAdminForm(ModelForm):
    """
    Base form for blog entries
    """
    def __init__(self, *args, **kwargs):
        super(AbstractEntryBaseAdminForm, self).__init__(*args, **kwargs)

        # The admin's .save() method fills in a default:
        self.fields['publication_date'].required = False
        try:
            author_field = self.fields['author']
        except KeyError:
            pass
        else:
            author_field.required = False
            self.initial.setdefault('author', author_field.user)

    def clean(self):
        cleaned_data = super(AbstractEntryBaseAdminForm, self).clean()
        if 'slug' not in cleaned_data or 'publication_date' not in cleaned_data:
            return cleaned_data

        try:
            self.validate_unique_slug(cleaned_data)
        except ValidationError as e:
            self._errors['slug'] = self.error_class(e.messages)

        return cleaned_data

    def clean_author(self):
        # Make sure the form never assigns None to the author value.
        return self.cleaned_data['author'] or self.fields['author'].user

    def validate_unique_slug(self, cleaned_data):
        """
        Test whether the slug is unique within a given time period.
        """
        date_kwargs = {}
        error_msg = _("The slug is not unique")

        # The /year/month/slug/ URL determines when a slug can be unique.
        pubdate = cleaned_data['publication_date'] or now()
        if '{year}' in appsettings.FLUENT_BLOGS_ENTRY_LINK_STYLE:
            date_kwargs['year'] = pubdate.year
            error_msg = _("The slug is not unique within it's publication year.")
        if '{month}' in appsettings.FLUENT_BLOGS_ENTRY_LINK_STYLE:
            date_kwargs['month'] = pubdate.month
            error_msg = _("The slug is not unique within it's publication month.")
        if '{day}' in appsettings.FLUENT_BLOGS_ENTRY_LINK_STYLE:
            date_kwargs['day'] = pubdate.day
            error_msg = _("The slug is not unique within it's publication day.")

        date_range = get_date_range(**date_kwargs)

        # Base filters are configurable for translation support.
        dup_filters = self.get_unique_slug_filters(cleaned_data)
        if date_range:
            dup_filters['publication_date__range'] = date_range

        dup_qs = EntryModel.objects.filter(**dup_filters)

        if self.instance and self.instance.pk:
            dup_qs = dup_qs.exclude(pk=self.instance.pk)

        # Test whether the slug is unique in the current month
        # Note: doesn't take changes to FLUENT_BLOGS_ENTRY_LINK_STYLE into account.
        if dup_qs.exists():
            raise ValidationError(error_msg)

    def get_unique_slug_filters(self, cleaned_data):
        # Allow to override this for translations
        return {
            'slug': cleaned_data['slug'],
        }



class AbstractTranslatableEntryBaseAdminForm(TranslatableModelForm, AbstractEntryBaseAdminForm):
    """
    Base form for translatable blog entries
    """
    def get_unique_slug_filters(self, cleaned_data):
        return {
            'translations__slug': cleaned_data['slug'],
            'translations__language_code': self.language_code
        }



class AbstractEntryBaseAdmin(PlaceholderFieldAdmin):
    """
    The base functionality of the admin, which only uses the fields of the
    :class:`~fluent_blogs.base_models.AbstractEntryBase` model.
    Everything else is branched off in the :class:`EntryAdmin` class.
    """
    list_display = ('title', 'status_column', 'modification_date', 'actions_column')
    list_filter = ('status',)
    date_hierarchy = 'publication_date'
    search_fields = ('slug', 'title')
    actions = ['make_published']
    form = AbstractEntryBaseAdminForm
    prepopulated_fields = {'slug': ('title',),}
    radio_fields = {
        'status': admin.HORIZONTAL,
    }
    raw_id_fields = ('author',)

    FIELDSET_GENERAL = (None, {
        'fields': ('title', 'slug', 'status',),
    })
    FIELDSET_PUBLICATION = (_('Publication settings'), {
        'fields': ('publication_date', 'publication_end_date', 'author'),
        'classes': ('collapse',),
    })


    class Media:
        css = {
            'all': ('fluent_blogs/admin/admin.css',)
        }


    def queryset(self, request):
        qs = super(AbstractEntryBaseAdmin, self).queryset(request)

        # Admin only shows current site for now,
        # until there is decent filtering for it.
        if appsettings.FLUENT_BLOGS_FILTER_SITE_ID:
            qs = qs.filter(parent_site=settings.SITE_ID)
        return qs

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = list(super(AbstractEntryBaseAdmin, self).get_readonly_fields(request, obj))
        if not request.user.is_superuser:
            readonly_fields.append('author')
        return readonly_fields

    def save_model(self, request, obj, form, change):
        # Automatically store the user in the author field.
        if not obj.author_id:
            obj.author = request.user

        if not obj.publication_date:
            # auto_now_add makes the field uneditable.
            # default fills the field before the post is written (too early)
            obj.publication_date = now()
        obj.save()


    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Allow formfield_overrides to contain field names too.
        """
        overrides = self.formfield_overrides.get(db_field.name)
        if overrides:
            kwargs.update(overrides)

        field = super(AbstractEntryBaseAdmin, self).formfield_for_dbfield(db_field, **kwargs)

        # Pass user to the form.
        if db_field.name == 'author':
            field.user = kwargs['request'].user
        return field


    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        # When the page is accessed via a pagetype, warn that the node can't be previewed yet.
        context['preview_error'] = ''
        if 'fluent_pages' in settings.INSTALLED_APPS:
            from fluent_pages.urlresolvers import PageTypeNotMounted, MultipleReverseMatch
            try:
                self._reverse_blogpage_index(request, obj)
            except PageTypeNotMounted:
                from fluent_blogs.pagetypes.blogpage.models import BlogPage
                context['preview_error'] = ugettext("The blog entry can't be previewed yet, a '{page_type_name}' page needs to be created first.").format(page_type_name=BlogPage._meta.verbose_name)
            except MultipleReverseMatch:
                # When 'entry_archive_index is ambiguous (because there are multiple blog nodes in the fluent-pages tree),
                # the edit page will automatically pick an option.
                pass
            except NoReverseMatch:
                # Since forgetting the pagetype app is easy, give off a warning to help developers
                # find their way with these apps.
                raise ImproperlyConfigured(
                    "To use django-fluent-blogs, either include('fluent_blogs.urls') in the URLConf, "
                    "or add the 'fluent_blogs.pagetypes.blogpage' app to the INSTALLED_APPS."
                )

        return super(AbstractEntryBaseAdmin, self).render_change_form(request, context, add, change, form_url, obj)


    def _reverse_blogpage_index(self, request, obj=None):
        # Internal method with "protected access" to handle translation differences.
        # This is only called when 'fluent_pages' is in the INSTALLED_APPS.
        from fluent_pages.urlresolvers import mixed_reverse
        return mixed_reverse('entry_archive_index')


    # ---- List code ----

    STATUS_ICONS = (
        (AbstractEntryBase.PUBLISHED, 'icon-yes.gif'),
        (AbstractEntryBase.DRAFT,     'icon-unknown.gif'),
    )


    @classmethod
    def get_status_column(cls, entry):
        # Create a status column, is also reused by templatetags/fluent_blogs_admin_tags.py
        status = entry.status
        title = next(rec[1] for rec in AbstractEntryBase.STATUSES if rec[0] == status)
        icon  = next(rec[1] for rec in cls.STATUS_ICONS if rec[0] == status)
        if django.VERSION >= (1, 4):
            admin = settings.STATIC_URL + 'admin/img/'
        else:
            admin = settings.ADMIN_MEDIA_PREFIX + 'img/admin/'
        return u'<img src="{admin}{icon}" width="10" height="10" alt="{title}" title="{title}" />'.format(admin=admin, icon=icon, title=title)


    def status_column(self, entry):
        # Method is needed because can't assign attributes to a class method
        return self.get_status_column(entry)

    status_column.allow_tags = True
    status_column.short_description = _('Status')


    @classmethod
    def get_actions_column(cls, entry):
        return u' '.join(cls._actions_column_icons(entry))


    @classmethod
    def _actions_column_icons(cls, entry):
        actions = []
        if cls.can_preview_object(entry):
            try:
                url = entry.get_absolute_url()
            except (NoReverseMatch, TranslationDoesNotExist):
                # A Blog Entry is already added, but the URL can no longer be resolved.
                # This can either mean that urls.py is missing a 'fluent_blogs.urls' (unlikely),
                # or that this is a PageTypeNotMounted exception because the "Blog page" node was removed.
                # In the second case, the edit page should still be reachable, and the "view on site" link will give an alert.
                pass
            else:
                actions.append(
                    u'<a href="{url}" title="{title}" target="_blank"><img src="{static}fluent_blogs/img/admin/world.gif" width="16" height="16" alt="{title}" /></a>'.format(
                        url=url, title=_('View on site'), static=settings.STATIC_URL)
                )
        return actions


    @classmethod
    def can_preview_object(cls, entry):
        """ Override whether the node can be previewed. """
        return hasattr(entry, 'get_absolute_url') and entry.is_published


    def actions_column(self, entry):
        return self.get_actions_column(entry)

    actions_column.allow_tags = True
    actions_column.short_description = _('Actions')


    def make_published(self, request, queryset):
        rows_updated = queryset.update(status=AbstractEntryBase.PUBLISHED)

        if rows_updated == 1:
            message = "1 entry was marked as published."
        else:
            message = "{0} entries were marked as published.".format(rows_updated)
        self.message_user(request, message)

    make_published.short_description = _("Mark selected entries as published")



class AbstractTranslatableEntryBaseAdmin(TranslatableAdmin, AbstractEntryBaseAdmin):
    """
    The base functionality of the admin, which only uses the fields of the
    :class:`~fluent_blogs.base_models.AbstractTranslatedEntryBase` model.
    Everything else is branched off in the :class:`EntryAdmin` class.
    """
    form = AbstractTranslatableEntryBaseAdminForm
    list_display = ('title', 'language_column', 'status_column', 'modification_date', 'actions_column')
    search_fields = ('translations__slug', 'translations__title')
    prepopulated_fields = {}  # Not supported by django-parler 0.9.2, using get_prepopulated_fields() as workaround.

    def get_prepopulated_fields(self, request, obj=None):
        # Still allow to override self.prepopulated_fields in other custom classes,
        # but default to the settings which are compatible with django-parler.
        return self.prepopulated_fields or {'slug': ('title',),}

    def _reverse_blogpage_index(self, request, obj=None):
        # Updated mixed_reverse() call, with language code included.
        from fluent_pages.urlresolvers import mixed_reverse
        language_code = self.get_form_language(request, obj)
        return mixed_reverse('entry_archive_index', language_code=language_code)

    def get_language_short_title(self, language_code):
        """
        Turn the language code to uppercase.
        """
        return language_code.upper()

    def changelist_view(self, request, extra_context=None):
        extra_context = extra_context or {}
        extra_context['FLUENT_BLOGS_IS_TRANSLATABLE'] = True
        return super(AbstractTranslatableEntryBaseAdmin, self).changelist_view(request, extra_context)


_model_fields = EntryModel._meta.get_all_field_names()
if issubclass(EntryModel, TranslatableModel):
    _entry_admin_base = AbstractTranslatableEntryBaseAdmin
    _model_fields += Entry_Translation.get_translated_fields()
else:
    _entry_admin_base = AbstractEntryBaseAdmin


class SeoEntryAdminMixin(object):
    """
    Mixin for the SEO fields.
    """
    FIELDSET_SEO = (_('SEO settings'), {
        'fields': ('meta_keywords', 'meta_description'),
        'classes': ('collapse',),
    })

    # AbstractEntryBaseAdmin allows to specify the widgets by field name,
    # which formfield_overrides doesn't support by default.
    formfield_overrides = {
        'meta_keywords': {
            'widget': AdminTextInputWidget(attrs={'class': 'vLargeTextField'})
        },
        'meta_description': {
            'widget': AdminTextareaWidget(attrs={'rows': 3})
        },
    }


class EntryAdmin(SeoEntryAdminMixin, _entry_admin_base):
    """
    The Django admin class for the default blog :class:`~fluent_blogs.models.Entry` model.
    When using a custom model, you can use :class:`AbstractEntryBaseAdmin`, which isn't attached to any of the optional fields.
    """
    # Redefine the fieldset, because it will be extended with auto-detected fields.
    FIELDSET_GENERAL = (None, {
        'fields': ('title', 'slug', 'status',),  # is filled with ('intro', 'contents', 'categories', 'tags', 'enable_comments') below
    })

    # For Django 1.4, the fieldsets shouldn't be declared with 'fieldsets ='
    # as the admin validation won't recognize the translated fields.
    # The 1.4 validation didn't check the form at all, but only checks the model fields.
    # As of Django 1.5, using 'fieldsets = ..' with translated fields just works.
    declared_fieldsets = (
        FIELDSET_GENERAL,
        AbstractEntryBaseAdmin.FIELDSET_PUBLICATION,
        SeoEntryAdminMixin.FIELDSET_SEO,
    )

    formfield_overrides = {}
    formfield_overrides.update(SeoEntryAdminMixin.formfield_overrides)
    formfield_overrides.update({
        'intro': {
            'widget': widgets.AdminTextareaWidget(attrs={'rows': 4})
        },
    })


# Add all fields
for _f in ('intro', 'contents', 'categories', 'tags', 'enable_comments'):
    if _f in _model_fields:
        EntryAdmin.FIELDSET_GENERAL[1]['fields'] += (_f,)



"""
Django compatibility features
"""
from django.conf import settings

__all__ = (
    'now', 'utc',
    'get_user_model', 'get_user_model_name',
    'patterns', 'url', 'include',
)


# The timezone support was introduced in Django 1.4, fallback to standard library for 1.3.
try:
    from django.utils.timezone import now, utc
except ImportError:
    # Django < 1.4
    from datetime import datetime
    now = datetime.now
    utc = None   # very naive


# Support for custom User models in Django 1.5+
try:
    from django.contrib.auth import get_user_model

    def get_user_model_name():
        return settings.AUTH_USER_MODEL
except ImportError:
    # django < 1.5
    from django.contrib.auth.models import User

    def get_user_model():
        return User

    def get_user_model_name():
        return '{0}.{1}'.format(User._meta.app_label, User._meta.object_name)


# URLs moved in Django 1.4
try:
    # Django 1.6 requires this
    from django.conf.urls import patterns, url, include
except ImportError:
    # Django 1.3 compatibility, kept in minor release
    from django.conf.urls.defaults import patterns, url, include



from django.conf import settings
from django.core.urlresolvers import reverse

_HAS_FLUENT_PAGES = ('fluent_pages' in settings.INSTALLED_APPS)
if _HAS_FLUENT_PAGES:
    from fluent_pages.urlresolvers import mixed_reverse


def blog_reverse(viewname, args=None, kwargs=None, current_app='fluent_blogs', current_page=None, language_code=None, multiple=False, ignore_multiple=False):
    """
    Reverse a URL to the blog, taking various configuration options into account.

    This is a compatibility function to allow django-fluent-blogs to operate stand-alone.
    Either the app can be hooked in the URLconf directly, or it can be added as a pagetype of *django-fluent-pages*.
    """
    if _HAS_FLUENT_PAGES:
        return mixed_reverse(viewname, args=args, kwargs=kwargs, current_page=current_page, language_code=language_code, multiple=multiple, ignore_multiple=ignore_multiple)
    else:
        return reverse(viewname, args=args, kwargs=kwargs, current_app=current_app)


__all__ = (
    'blog_reverse',
)
